/*****************************************************************************
 *  Rohde&Schwarz CMW Radiocommunication Analyzer Base instrument driver
 *  VXIpnp Instrument Driver
 *  Original Release: October 2007
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Version: 3.5.1000
 *
 *  Modification History:
 *     see file ChangeLog
 *****************************************************************************/
#include <math.h>
#include <float.h>
#include "rscmw.h"
#include "rscmw_utility.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

/*****************************************************************************
 *****************************************************************************/

/*****************************************************************************
 * INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================
*****************************************************************************/
static ViString statusRegisterArr[] = {"COND", "EVEN", "ENAB", "PTR", "NTR", NULL};
static ViString statusTypeOfQueryArr[] = {"ALL", "COUN", "NEXT", "CAT", NULL};
static ViString statusFncGrpArr[] = {"MEAS", "GEN", NULL};
static ViString statusFncTypeOfQueryArr[] = {"OFF", "ON", "PNED", "QUED", "RDY", "RUN", "SDR", NULL};
static ViString attDirectionArr[] = {"RX", "TX", "RXTX", NULL};
static ViString attConnectorCMDArr[] = {"RF1O", "RF1C", "RF2C", "RF3O", "RF3C", "RF4C", "RFAO", "RFAC", "RFBC", NULL};
static ViString networkSegmentArr[] = {"A", "B", "C", NULL};
static ViString rfConverterArr[] = {"RF1", "RF2", "RF3", "RF4", NULL};
static ViString attDirectionCMWSArr[] = {"RX", "TX", NULL};
static ViString attConnectorCMWSArr[] = {
	"R11", "R12", "R13", "R14", "R15", "R16", "R17", "R18",
	"R21", "R22", "R23", "R24", "R25", "R26", "R27", "R28",
	"R31", "R32", "R33", "R34", "R35", "R36", "R37", "R38",
	"RA1", "RA2", "RA3", "RA4", "RA5", "RA6", "RA7", "RA8",
	"RB1", "RB2", "RB3", "RB4", "R118", "R214", "R318",
	"R258", "RA18", "RB14", NULL
};
static ViString offOnArr[] = {"OFF", "ON", NULL};
static ViString statusArr[] = {"NADJ", "ADJ", NULL};
static ViString attDirectionTablesArr[] = {"RX:ALL", "TX:ALL", "ALL", NULL};

/*****************************************************************************
 * INSTRUMENT-DEPENDENT ATTRIBUTE ARRAYS ===================================
*****************************************************************************/
static ViString statusRegisterOperArr[] = {"", "reg_quest", "reg_oper", NULL};
static ViString statusRegisterPartArr[] = {"part_condition", "part_event", "part_enable", "part_ptr", "part_ntr", NULL};
static ViString statusRegisterPartSimpleArr[] = {"part_enable", "part_ntr", "part_ptr", NULL};
static ViString statusRegisterPartQSimpleArr[] = {"part_condition", "part_event", NULL};
static ViString statusRegisterInstrPartArr[] = {"", "part_a", "part_b", NULL};
static ViString statusRegisterInstanceArr[] = {"", "i1", "i2", "i3", "i4", NULL};
static ViString statusRegisterNetStdPartArr[] = {"", "netw", "netwGSM", "netwWCDMA", "netwEVDO", "netwCDMA", "netwTDSCDMA", NULL};
static ViString statusRegisterFncGrpPartArr[] = {"func_meas", "func_gen", NULL};
static ViString statusRegisterAppPartArr[] = {"appl_uni", "appl_mev", "appl_eps", "appl_pow", "appl_iqv", "appl_iqr", NULL};
static ViString statusRegisterBitsArr[] = {"bits8", "bits9", "bits10", "bits11", "bits12", NULL};
static ViString statusConnectorArr[] = {"con_a", "con_b", NULL};
static ViString presetPartArr[] = {"rst_cmw", "rst_base", "rst_all", NULL};

/*****************************************************************************
*- Callback Declarations ---------------------------------------------------*
*****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rscmw_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rscmw_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_init(ViRsrc resourceName, ViBoolean IDQuery,
                             ViBoolean resetDevice, ViSession *newInstrSession)
{
	ViStatus error = VI_SUCCESS;

	if (newInstrSession == NULL)
	{
		(void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
		                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

		checkErr(RS_ERROR_INVALID_PARAMETER);
	}

	checkErr(rscmw_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
	return error;
}

/*****************************************************************************
 * Function: rscmw_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_InitWithOptions(
	ViRsrc resourceName,
	ViBoolean IDQuery,
	ViBoolean resetDevice,
	ViString optionString,
	ViSession *newInstrSession
)
{
	ViStatus error = VI_SUCCESS;
	RsCoreAttributePtr *attrList = NULL;
	ViSession instrSession = 0;
	RsCoreSessionPtr sessionProperties = NULL;

	if (newInstrSession == NULL)
	{
		(void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
		                          "Null address for Instrument Handle");
		checkErr(RS_ERROR_INVALID_PARAMETER);
	}

	attrList = (RsCoreAttributePtr *)rscmw_RsCoreAttributeList;

	*newInstrSession = 0;

	checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rscmw", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        100000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSCMW_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

	// Minimal allowed rscore version is 3.4.0
	checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 3, 4, 0));

	checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

	// No SCPI command has been sent yet.
	// Call viClear before sending any SCPI command
	checkErr(RsCore_ViClear(instrSession));

	/* --- Here perform settings that are default for this driver,
	but can be overwritten by the optionsString settings */

	sessionProperties->autoSystErrQuery = VI_FALSE;

	// Parse option string and optionally sets the initial state of the following session attributes
	checkErr(RsCore_ApplyOptionString(instrSession, optionString));
	checkErr(RsCore_BuildRepCapTable(instrSession, rscmw_RsCoreRepCapTable));

	// Query *IDN? string, parse it and set the following attributes:
	// -RS_ATTR_INSTRUMENT_MANUFACTURER
	// - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "CMW100A", false : "CMW"
	// - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
	checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSCMW_SIMULATION_ID_QUERY, NULL));

	// Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
	checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSCMW_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

	// Default Instrument Setup + optional *RST
	if (resetDevice == VI_TRUE)
	{
		checkErr(rscmw_reset(instrSession));
	}
	else
	{
		checkErr(rscmw_DefaultInstrSetup(instrSession));
	}

	if (IDQuery == VI_TRUE)
	{
		checkErr(RsCore_FitsIDNpattern(instrSession, RSCMW_VALID_ID_RESPONSE_STRING, NULL));
	}

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	if (error < VI_SUCCESS)
	{
		if (error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties->autoSystErrQuery == VI_FALSE)
		{
			sessionProperties->autoSystErrQuery = VI_TRUE;
			(void)rscmw_CheckStatus(instrSession);
			sessionProperties->autoSystErrQuery = VI_FALSE;
		}

		if (instrSession != 0)
			(void)RsCore_ViClose(instrSession);
	}
	else
	{
		*newInstrSession = instrSession;
	}

	return error;
}

/// HIFN  This function switches off all generators. Same effect like pressing
/// HIFN  ON | OFF for all generators currently generating a signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_AllGeneratorsOff(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_ALL_GENERATORS_OFF, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function switches off all measurements. Same effect like pressing
/// HIFN  ON | OFF for all measurements currently switched on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_AllMeasurementsOff(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_ALL_MEASUREMENTS_OFF, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function switches off all signaling applications. Same effect
/// HIFN  like pressing ON | OFF for all signaling applications currently
/// HIFN  generating a signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_AllSignalingApplicationsOff(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_ALL_SIGNALING_APPLICATIONS_OFF, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Configure Reference Oscillator
 * Purpose:  This function selects the source of the reference oscillator
 *           signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureReferenceOscillator(ViSession instrSession,
                                                     ViInt32   reference,
                                                     ViReal64  externalFrequency)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_SYSTEM_REFERENCE_FREQUENCY_SOURCE, reference),
			2, "Reference");
	if (reference == RSCMW_VAL_EXT)
	{
		viCheckParm(rscmw_SetAttributeViReal64(instrSession, "", RSCMW_ATTR_SYSTEM_REFERENCE_EXTERNAL_FREQUENCY, externalFrequency),
				3, "External Frequency");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: Get Reference Frequency State
 * Purpose:  This function queries whether the reference frequency is locked
 *           or not. A not locked reference frequency is also indicated via
 *           the red "ERROR" LED at the front panel.
 *****************************************************************************/

ViStatus _VI_FUNC rscmw_GetReferenceFrequencyState(ViSession instrSession,
                                                   ViBoolean *locked)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_GetAttributeViBoolean(instrSession, "", RSCMW_ATTR_REFERENCE_FREQUENCY_STATE, locked),
			2, "Locked");

Error:
	return error;
}


/// HIFN Specifies the role of the instrument in a multi-CMW setup,
/// HIFN concerning the system time synchronization signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR mode/Specifies the role of the instrument in a multi-CMW setup,
/// HIPAR mode/concerning the system time synchronization signal.
ViStatus _VI_FUNC rscmw_ConfigureSynchronizationMode(ViSession instrSession,
                                                     ViInt32   mode)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_SYNCHRONIZATION_MODE, mode),
			2, "Mode");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_LockControls
 * Purpose:  Locks or unlocks the local controls of the instrument. This
 *           includes the front panel keys, the keyboard, or other local
 *           interfaces.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_LockControls(ViSession instrSession,
                                     ViBoolean locked)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_CONFIGURE_LOCK_CTRLS, locked),
			2, "Configure Lock Controls");

Error:
	return error;
}

/// HIFN  Returns a list of installed software options (licenses), hardware options, software packages and firmware applications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR optionType/List only software options, hardware options, software packages or firmware applications.
/// HIPAR validity/List only functional entries or only valid entries.
/// HIPAR optionList/Returns a list of installed software options (licenses),
/// HIPAR optionList/hardware options, software packages and firmware applications.
ViStatus _VI_FUNC rscmw_QueryOptionList(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViInt32   optionType,
                                        ViInt32   validity,
                                        ViChar    optionList[])
{
	ViStatus error = VI_SUCCESS;
	ViString optionTypeArr[] = {"ALL", "SWOP", "HWOP", "SWP", "FWA"};
	ViString validityArr[] = {"ALL", "VAL", "FUNC"};
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, optionType, 0, 4),
			2, "Option Type");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, validity, 0, 2),
			3, "Validity");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:BASE:OPT:LIST? %s,%s", optionTypeArr[optionType], validityArr[validity]);
	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, optionList, NULL));
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns version information for installed software packages. The
/// HIFN  "Setup"dialog provides this information in section "SW/HW-Equipment >
/// HIFN  Installed Software". You can either query a list of all installed
/// HIFN  packages and their versions or you can query the version of a single
/// HIFN  package specified via parameter <Application>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Value parameter.
/// HIPAR application/String selecting the software package for which the version shall be
/// HIPAR application/queried.
/// HIPAR optionList/String containing a single version or a list of applications and
/// HIPAR optionList/versions.
ViStatus _VI_FUNC rscmw_QueryOptionVersionInfo(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViString  application,
                                               ViChar    optionList[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	if (strlen(application) > 0)
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:BASE:OPT:VERS? '%s'", application);
	else
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:BASE:OPT:VERS?");

	checkErr(rscmw_QueryViString(instrSession, cmd, bufferSize, optionList));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns the common reliability indicator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR reliability/Returns the common reliability indicator.
ViStatus _VI_FUNC rscmw_QueryCommonReliability(ViSession instrSession,
                                               ViInt32   *reliability)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViInt32(instrSession, "", RSCMW_ATTR_COMMON_RELIABILITY, reliability),
			2, "Reliability");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Queries the relation between absolute and virtual connector names.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR connector/Selects the connector.
/// HIPAR virtualConnector/Returns the virtual connector.
/// HIPAR absoluteConnector/Returns the absolute connector.
ViStatus _VI_FUNC rscmw_QueryConnectorRelation(ViSession instrSession,
                                               ViInt32   connector,
                                               ViInt32   *virtualConnector,
                                               ViInt32   *absoluteConnector)
{
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, 0, 8),
			2, "Connector");
	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:CONN:TRAN? %s", attConnectorCMDArr[connector]);
	checkErr(RsCore_QueryTupleViInt32(instrSession, cmd, virtualConnector, absoluteConnector, NULL, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Queries the temperature within the instrument. The returned value
/// HIFN      indicates the average of the temperatures measured at the individual RF
/// HIFN      modules.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR internalTemperature/The returned value indicates the average of the temperatures measured
/// HIFN      at the individual RF modules.
ViStatus _VI_FUNC rscmw_QueryInternalTemperature(ViSession instrSession,
                                                 ViReal64  *internalTemperature)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViReal64(instrSession, "", RSCMW_ATTR_INTERNAL_TEMPERATURE, internalTemperature),
			2, "Internal Temperature");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the logging PC address pool. In the signaling application
/// HIFN  you select the pool entry to be used, so different signaling
/// HIFN  applications can use different logging PCs. The logging PCs are
/// HIFN  connected to the internal subnet of the instrument. Thus the first two
/// HIFN  address segments are determined by the subnet configuration. The other
/// HIFN  parts sup-port only a limited range. Assign compatible addresses to the
/// HIFN  logging PCs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR poolEntry/Selects the pool entry to be configured.
/// HIPAR firstSegment/First octet of the IP address, not configurable.
/// HIPAR secondSegment/Second octet of the IP address, not configurable.
/// HIPAR nodeID/Third octet of the IP address.
/// HIPAR localID/Fourth octet of the IP address, not configurable.
ViStatus _VI_FUNC rscmw_ConfigureSignalingMessageMonitoring(ViSession instrSession,
                                                            ViInt32   poolEntry,
                                                            ViInt32   firstSegment,
                                                            ViInt32   secondSegment,
                                                            ViInt32   nodeID,
                                                            ViInt32   localID)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:MMON:IPAD%ld %ld,%ld,%ld,%ld", poolEntry, firstSegment,
	         secondSegment, nodeID, localID);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Selects a fan control mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fanControl/Selects a fan control mode.
ViStatus _VI_FUNC rscmw_ConfigureFanControl(ViSession instrSession,
                                            ViInt32   fanControl)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_FAN_CONTROL, fanControl),
			2, "Fan Control");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Enables or disables the multiple-window mode of the graphical user
/// HIFN      interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR enabled/Enables or disables the multiple-window mode of the graphical user
/// HIFN      interface.
ViStatus _VI_FUNC rscmw_ConfigureMultipleWindowMode(ViSession instrSession,
                                                    ViBoolean enabled)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_MULTIPLE_WINDOW_MODE_ENABLED, enabled),
			2, "Enabled");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: Configure Adapter
 * Purpose:  Selects the network adapter and thus the connection type. At
 *           present, LAN connections are supported.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureAdapter(ViSession instrSession,
                                         ViString  networkAdapter)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CONFIGURE_NETWORK_ADAPTER, networkAdapter));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Configure Hostname
 * Purpose:  Selects the host name (computer name) of the R&S CMW 500. The
 *           host name (without the domain) is part of the VISA address
 *           string for LAN-based connections.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureHostname(ViSession instrSession,
                                          ViString  hostname)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CONFIGURE_NETWORK_HOSTNAME, hostname));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Configure DHCP
 * Purpose:  Enables or disables Dynamic Host Configuration Protocol (DHCP).
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureDHCP(ViSession instrSession,
                                      ViBoolean DHCPState)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_CONFIGURE_NETWORK_DHCP, DHCPState));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Enable DNS Address Configuration
 * Purpose:  Enables or disables dynamic DNS address configuration.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_EnableDNSAddressConfiguration(ViSession instrSession,
                                                      ViBoolean DNSState)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_ENABLE_DNS_ADDRESS_CONFIGURATION, DNSState));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Configure DNS Address
 * Purpose:  Defines the preferred and the alternate DNS addresses of the R&S
 *           CMW 500. The addresses are valid if dynamic DNS address
 *           configuration is disabled.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureDNSAddress(ViSession instrSession,
                                            ViString  preferredAddress,
                                            ViString  alternateAddress)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:NET:DNS '%s','%s'", preferredAddress, alternateAddress);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);

	return error;
}


/*****************************************************************************
 * Function: Configure IP Address
 * Purpose:  This function defines the IP address, subnet mask or gateway of
 *           the R&S CMW 500.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureIPAddress(ViSession instrSession,
                                           ViInt32   ipAdressSetting,
                                           ViString  ipAddress)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, ipAdressSetting, 0, 1),
			2, "IP Address Setting");

	switch (ipAdressSetting)
	{
	case RSCMW_VAL_IP_INSTR:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CONFIGURE_NETWORK_IP_ADDRESS, ipAddress));
		break;
	case RSCMW_VAL_IP_MASK:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CONFIGURE_NETWORK_SUBNET_MASK, ipAddress));
		break;
	case RSCMW_VAL_IP_GATEWAY:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CONFIGURE_NETWORK_GATEWAY, ipAddress));
		break;
	}
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);

	return error;
}


/*============================================================================/
 * Function: Set GPIB Address
 * Purpose:  This function selects the GPIB address of the analyzer.
/============================================================================*/

ViStatus _VI_FUNC rscmw_SetGPIBAddress(ViSession instrSession,
                                       ViInt32   address)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_SYSTEM_GPIB_ADDRESS, address),
			2, "Address");

Error:
	return error;
}


/*============================================================================/
 * Function: Enable Interface
 * Purpose:  Enables or disables the instrument interface.
/============================================================================*/
ViStatus _VI_FUNC rscmw_EnableInterface(ViSession instrSession,
                                        ViInt32   intfc,
                                        ViBoolean interfaceState)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, intfc, 0, 1),
			2, "Interface");

	switch (intfc)
	{
	case RSCMW_VAL_INSTR_GPIB:
		viCheckParm(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_SYSTEM_ENABLE_GPIB_INTERFACE, interfaceState),
				3, "Interface State");
		break;
	case RSCMW_VAL_INSTR_VXI:
		viCheckParm(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_SYSTEM_ENABLE_VXI_INTERFACE, interfaceState),
				3, "Interface State");
		break;
	}

Error:
	return error;
}


/*============================================================================/
 * Function: Configure Font
 * Purpose:  This function selects the font size for the GUI labels.
/============================================================================*/
ViStatus _VI_FUNC rscmw_ConfigureFont(ViSession instrSession,
                                      ViInt32   fontSize)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "fontsize", RSCMW_ATTR_CONFIGURE_FONT_SIZE, fontSize),
			2, "Font size");

Error:
	return error;
}


/*============================================================================/
 * Function: Configure Update On Remote
 * Purpose:  This function defines whether the display shall be updated or not
 * while the instrument is in the remote state. If the display (update) is
 * switched off, the normal GUI is replaced by a static image while
 * the instrument is in the remote state. Switching off the display
 * can speed up the measurement. This is the recommended state.
/============================================================================*/
ViStatus _VI_FUNC rscmw_ConfigureUpdateOnRemote(ViSession instrSession,
                                                ViBoolean update)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_CONFIGURE_UPDATE_ON_REMOTE, update),
			2, "Configure Update On Remote");

Error:
	return error;
}


/// HIFN Turns the built-in display, the external monitor on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR turnOffMonitor/Enables or disables the built-in display, the external monitor on or off.
ViStatus _VI_FUNC rscmw_ConfigureMonitorDisplayState(ViSession instrSession,
                                                     ViBoolean turnOffMonitor)
{
	ViStatus error = VI_SUCCESS;

	checkErr(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_MONITOR_DISPLAY_STATE, turnOffMonitor));

Error:
	return error;
}

/// HIFN Selects the tree traversal mode of the rotary knob.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR treeNavigationMode/Selects the tree traversal mode of the rotary knob.
ViStatus _VI_FUNC rscmw_ConfigureTreeNavigationMode(ViSession instrSession,
                                                    ViInt32   treeNavigationMode)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "",
        RSCMW_ATTR_TREE_NAVIGATION_MODE, treeNavigationMode), 2, "treeNavigationMode")

Error:
	return error;
}

/*****************************************************************************
 * Function: rscmw_ConfigureSubinstruments
 * Purpose:  This function selects the number of sub-instruments. The physical
 *           instrument can be split into two sub-instruments or all resources
 *           can be assigned to a single sub-instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureSubinstruments(ViSession instrSession,
                                                ViInt32   subinstruments)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_CONFIGURE_SUBINSTRUMENTS, subinstruments),
			2, "Configure Subinstruments");

Error:
	return error;
}

/// HIFN  Assigns the available hardware resources to the subinstruments.
/// HIFN     Send this command to the subinstrument with the lowest number (device
/// HIFN      number 0 /assigned instrument 1 / subinstrument 1).
/// HIFN     After changing the number of subinstruments via
/// HIFN      SYSTem:BASE:DEVice:COUNt, always send this command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_AssignResourceToSubinstruments(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_ASSIGN_RESOURCES_TO_SUBINSTRUMENTS, NULL),
			2, "Assign Resources To Subinstruments");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Queries the number of the addressed sub-instrument and the total number of sub-instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR currentSubinstrument/Number of the addressed sub-instrument, as indicated in a VISA resource string for VXI-11.
/// HIPAR subinstrumentsCount/Total number of sub-instruments.
ViStatus _VI_FUNC rscmw_GetSubinstrumentsInfo(ViSession instrSession,
                                              ViInt32   *currentSubinstrument,
                                              ViInt32   *subinstrumentsCount)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_QueryTupleViInt32(instrSession, "SYST:BASE:DEV:SUB?", currentSubinstrument, subinstrumentsCount, NULL, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns the maximum number of subinstruments into which the instrument
/// HIFN      can be split.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR subinstrumentsCount/Returns the maximum number of subinstruments into which the instrument
/// HIFN      can be split.
ViStatus _VI_FUNC rscmw_GetMaxNumberOfSubinstruments(ViSession instrSession,
                                                     ViInt32   *subinstrumentsCount)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViInt32(instrSession, "", RSCMW_ATTR_MAX_NUMBER_OF_SUBINSTRUMENTS, subinstrumentsCount),
			2, "Sub-instruments Count");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Queries the device identification (independent of sub-instruments).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR deviceID/Returns he device identification (independent of sub-instruments).
ViStatus _VI_FUNC rscmw_QueryDeviceID(ViSession instrSession,
                                      ViInt32   bufferSize,
                                      ViChar    deviceID[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_DEVICE_ID,
			bufferSize, deviceID),
			3, "Device ID");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Queries the device identification of an CMWS connected to the CMW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR deviceID/Queries the device identification of an CMWS connected to the CMW.
ViStatus _VI_FUNC rscmw_QueryCMWSDeviceID(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    deviceID[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_CMWS_DEVICE_ID,
			bufferSize, deviceID),
			3, "Device ID");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Sets the 'Device Group' that the instrument belongs to. For remote
/// HIFN      installation, this setting must match the corresponding setting in the
/// HIFN      R&S Software Distributor options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR subinstruments/Sets the 'Device Group' that the instrument belongs to. For remote
/// HIFN      installation, this setting must match the corresponding setting in the
/// HIFN      R&S Software Distributor options.
ViStatus _VI_FUNC rscmw_ConfigureDeviceGroup(ViSession instrSession,
                                             ViString  subinstruments)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_DEVICE_GROUP, subinstruments),
			2, "Sub-instruments");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Enables startup with factory default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bootFactoryState/Enables startup with factory default state.
ViStatus _VI_FUNC rscmw_ConfigureStartupBehavior(ViSession instrSession,
                                                 ViBoolean bootFactoryState)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_SetAttributeViBoolean(instrSession, "", RSCMW_ATTR_STARTUP_BEHAVIOR, bootFactoryState),
			2, "Boot Factory State");

Error:
	return error;
}

/*****************************************************************************
* Function: rscmw_clearEventStatusRegister
* Purpose:  This function clears the EVENt parts of all status registers of
*           the status operation register hierarchy. If a regular expression
*           is defined, the command is only applied to the registers matching
*           the filter criteria.
*****************************************************************************/
ViStatus _VI_FUNC rscmw_ClearEventStatusRegister(ViSession instrSession,
                                                 ViString  regular_expression)
{
	ViStatus error = VI_SUCCESS;

	if (regular_expression == NULL)
	{
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CLEAR_ALL_EVENT_STATUS_REGISTER, regular_expression));
	}
	else
	{
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CLEAR_EVENT_STATUS_REGISTER, regular_expression),
				2, "Clear Event Status Register");
	}

Error:
	return error;
}

/*============================================================================/
 * Function: Set Status Register
 * Purpose:  This function sets the Enable, NTransition, and PTransition bits
 *           of status questionable and operating registers.
/============================================================================*/

ViStatus _VI_FUNC rscmw_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   enable,
                                          ViInt32   PTransition,
                                          ViInt32   NTransition)
{
	ViStatus error = VI_SUCCESS;

	// *U not used, use atribute function

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerOperation, RSCMW_VAL_PRESET_STAT_REG, RSCMW_VAL_OPER_STAT_REG),
			2, "Register Operation");

	switch (registerOperation)
	{
	case RSCMW_VAL_PRESET_STAT_REG:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_PRESET_STATUS_REGISTERS, NULL));
		break;
	case RSCMW_VAL_QUEST_STAT_REG:
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "reg_quest,part_enable", RSCMW_ATTR_STATUS_OQ_CONFIGURATION, enable),
				3, "Status OQ Configuration");

		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "reg_quest,part_ntr", RSCMW_ATTR_STATUS_OQ_CONFIGURATION, NTransition),
				3, "Status OQ NTransition");

		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "reg_quest,part_ptr", RSCMW_ATTR_STATUS_OQ_CONFIGURATION, PTransition),
				3, "Status OQ PTransition");
		break;
	case RSCMW_VAL_OPER_STAT_REG:
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "reg_oper,part_enable", RSCMW_ATTR_STATUS_OQ_CONFIGURATION, enable),
				3, "Status OQ Configuration");

		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "reg_oper,part_ntr", RSCMW_ATTR_STATUS_OQ_CONFIGURATION, NTransition),
				3, "Status OQ NTransition");

		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "reg_oper,part_ptr", RSCMW_ATTR_STATUS_OQ_CONFIGURATION, PTransition),
				3, "Status OQ PTransition");
		break;
	}

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*============================================================================/
 * Function: Get Status Register
 * Purpose:  This function contains the commands for the querying status
 *           reporting system.
/============================================================================*/
ViStatus _VI_FUNC rscmw_GetStatusRegister(ViSession instrSession,
                                          ViInt32   registerOperation,
                                          ViInt32   statusRegistersQuery,
                                          ViInt32   *registerValue)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerOperation, RSCMW_VAL_QUEST_STAT_REG, RSCMW_VAL_OPER_STAT_REG),
			2, "Register Operation");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, statusRegistersQuery, RSCMW_VAL_REG_CONDITION, RSCMW_VAL_REG_NTRANSITION),
			3, "Status Registers Query");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", statusRegisterOperArr[registerOperation], statusRegisterPartArr[statusRegistersQuery]);
	viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_CONFIGURATION, registerValue),
			4, "Register Value");

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: rscmw_SetStatusRegisterTask
 * Purpose:  This function contains the commands for the seting status task
 *           reporting system.
 *
 *           Notes:
 *           (1) For detailed description of Status Reporting Task System see
 *           also operating manual.
 *
 *           (2)
 *           The allowed combinations of the listed variables correspond to
 *           the installed firmware applications.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_SetStatusRegisterTask(ViSession instrSession,
                                              ViInt32   registerPart,
                                              ViInt32   instrumentPart,
                                              ViInt32   networkStandard,
                                              ViInt32   subsystem,
                                              ViInt32   functionGroup,
                                              ViInt32   functionGroupInstance,
                                              ViInt32   appliacation,
                                              ViInt32   registerValue)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, RSCMW_VAL_REG_PART_S_ENAB, RSCMW_VAL_REG_PART_S_PTR),
			2, "Register Part");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, instrumentPart, RSCMW_VAL_INSTR_PART_A, RSCMW_VAL_INSTR_PART_A),
			3, "Instrument Part");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, networkStandard, RSCMW_VAL_TASK_NET_GPRF, RSCMW_VAL_TASK_NET_TDSC),
			4, "Network Standard");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, subsystem, RSCMW_VAL_REG_OPER, RSCMW_VAL_REG_TASK_A_APP),
			5, "Subsystem");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionGroup, RSCMW_VAL_TASK_GRP_MEAS, RSCMW_VAL_TASK_GRP_GEN),
			6, "Function Group");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionGroupInstance, 1, 2),
			7, "Function Group Instanece");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, appliacation, RSCMW_VAL_TASK_APP_UNI, RSCMW_VAL_TASK_APP_IQR),
			8, "Application");

	switch (subsystem)
	{
	case RSCMW_VAL_REG_OPER:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", statusRegisterOperArr[2], statusRegisterPartSimpleArr[registerPart]);
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_CONFIGURATION, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", statusRegisterOperArr[2], statusRegisterPartSimpleArr[registerPart]);
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_CONFIGURATION, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterPartSimpleArr[registerPart]);
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_INSTRUMENT_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A_NET:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterNetStdPartArr[networkStandard], statusRegisterPartSimpleArr[registerPart]);
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_NETSTD_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A_FUN:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s,%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterNetStdPartArr[networkStandard], statusRegisterFncGrpPartArr[functionGroup],
		         statusRegisterInstanceArr[functionGroupInstance], statusRegisterPartSimpleArr[registerPart]);
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_FUNCTION_GROUP_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A_APP:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s,%s,%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterNetStdPartArr[networkStandard], statusRegisterFncGrpPartArr[functionGroup],
		         statusRegisterInstanceArr[functionGroupInstance],
		         statusRegisterAppPartArr[appliacation], statusRegisterPartSimpleArr[registerPart]);
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	}
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: rscmw_GetStatusRegisterTask
 * Purpose:  This function contains the commands for the querying status task
 *           reporting system.
 *
 *           Notes:
 *           (1) For detailed description of Status Reporting Task System see
 *           also operating manual.
 *
 *           (2)
 *           The allowed combinations of the listed variables correspond to
 *           the installed firmware applications.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetStatusRegisterTask(ViSession instrSession,
                                              ViInt32   registerPart,
                                              ViInt32   instrumentPart,
                                              ViInt32   networkStandard,
                                              ViInt32   subsystem,
                                              ViInt32   functionGroup,
                                              ViInt32   functionGroupInstance,
                                              ViInt32   appliacation,
                                              ViInt32   *registerValue)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, RSCMW_VAL_REG_PART_COND, RSCMW_VAL_REG_PART_EVENT),
			2, "Register Part");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, instrumentPart, RSCMW_VAL_INSTR_PART_A, RSCMW_VAL_INSTR_PART_A),
			3, "Instrument Part");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, networkStandard, RSCMW_VAL_TASK_NET_GPRF, RSCMW_VAL_TASK_NET_TDSC),
			4, "Network Standard");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, subsystem, RSCMW_VAL_REG_OPER, RSCMW_VAL_REG_TASK_A_APP),
			5, "Subsystem");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionGroup, RSCMW_VAL_TASK_GRP_MEAS, RSCMW_VAL_TASK_GRP_GEN),
			6, "Function Group");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionGroupInstance, 1, 2),
			7, "Function Group Instanece");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, appliacation, RSCMW_VAL_TASK_APP_UNI, RSCMW_VAL_TASK_APP_IQR),
			8, "Application");

	switch (subsystem)
	{
	case RSCMW_VAL_REG_OPER:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", statusRegisterOperArr[2], statusRegisterPartQSimpleArr[registerPart]);
		viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_CONFIGURATION, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", statusRegisterOperArr[2], statusRegisterPartQSimpleArr[registerPart]);
		viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_CONFIGURATION, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterPartQSimpleArr[registerPart]);
		viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_INSTRUMENT_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A_NET:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterNetStdPartArr[networkStandard], statusRegisterPartQSimpleArr[registerPart]);
		viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_NETSTD_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A_FUN:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s,%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterNetStdPartArr[networkStandard], statusRegisterFncGrpPartArr[functionGroup],
		         statusRegisterInstanceArr[functionGroupInstance], statusRegisterPartQSimpleArr[registerPart]);
		viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_FUNCTION_GROUP_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	case RSCMW_VAL_REG_TASK_A_APP:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s,%s,%s,%s,%s", statusRegisterOperArr[2], statusRegisterInstrPartArr[instrumentPart],
		         statusRegisterNetStdPartArr[networkStandard], statusRegisterFncGrpPartArr[functionGroup],
		         statusRegisterInstanceArr[functionGroupInstance],
		         statusRegisterAppPartArr[appliacation], statusRegisterPartQSimpleArr[registerPart]);
		viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_TASK_PART_CONFIGURE, registerValue),
				9, "Register Value");
		break;
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_GetStatusRegisterAllBitsTasks
 * Purpose:  This function contains the commands for the queries status
 *           reporting system all tasks.
 *
 *           Notes:
 *           (1) Reg Exp
 *           Optional regular expression filtering the returned results
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetStatusRegisterAllBitsTasks(ViSession instrSession,
                                                      ViInt32   registerPart,
                                                      ViInt32   typeOfQuery,
                                                      ViString  regular_expression,
                                                      ViInt32   bufferSize,
                                                      ViChar    state[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, RSCMW_VAL_REG_PART_COND, RSCMW_VAL_REG_PART_EVENT),
			2, "Register Part");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, typeOfQuery, RSCMW_VAL_REG_OV_ALL, RSCMW_VAL_REG_OV_CAT),
			3, "Type Of Query");

	if (regular_expression == NULL)
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:BITS:%s?", statusRegisterArr[registerPart], statusTypeOfQueryArr[typeOfQuery]);
	}
	else
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:BITS:%s? '%s'", statusRegisterArr[registerPart],
		         statusTypeOfQueryArr[typeOfQuery], regular_expression);
	}

	checkErr(rscmw_QueryViString(instrSession,cmd,bufferSize,state));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: rscmw_GetStatusRegisterLowestTasks
 * Purpose:  This function contains the commands for the queries status
 *           reporting system lowest tasks.
 *
 *           Notes:
 *           (1) RegExp
 *           Optional regular expression filtering the returned results
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetStatusRegisterLowestTasks(ViSession instrSession,
                                                     ViInt32   functionGroup,
                                                     ViInt32   typeOfQuery,
                                                     ViString  regular_expression,
                                                     ViInt32   bufferSize,
                                                     ViChar    state[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, functionGroup, RSCMW_VAL_TASK_GRP_MEAS, RSCMW_VAL_TASK_GRP_GEN),
			2, "Function Group");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, typeOfQuery, RSCMW_VAL_REG_OV_OFF, RSCMW_VAL_REG_OV_SDR),
			3, "Type Of Query");

	if (regular_expression == NULL)
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:COND:%s?", statusFncGrpArr[functionGroup], statusFncTypeOfQueryArr[typeOfQuery]);
	}
	else
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:COND:%s? '%s'", statusFncGrpArr[functionGroup],
		         statusFncTypeOfQueryArr[typeOfQuery], regular_expression);
	}

	checkErr(rscmw_QueryViString(instrSession,cmd,bufferSize,state));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: rscmw_SetStatusRegisterBit
 * Purpose:  This function contains the commands for the seting status bit
 *           reporting system.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_SetStatusRegisterBit(ViSession instrSession,
                                             ViInt32   registerOperation,
                                             ViInt32   registerPart,
                                             ViInt32   bitNumber,
                                             ViBoolean bitValue)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerOperation, RSCMW_VAL_QUEST_STAT_REG, RSCMW_VAL_OPER_STAT_REG),
			2, "Register Operation");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, RSCMW_VAL_REG_PART_S_ENAB, RSCMW_VAL_REG_PART_S_PTR),
			3, "Register Part");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, bitNumber, 8, 12),
			4, "bit Number");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", statusRegisterOperArr[registerOperation], statusRegisterBitsArr[bitNumber - 8],
	         statusRegisterPartSimpleArr[registerPart]);

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_REGISTER_BIT_CONFIGURATION, (ViInt32)bitValue), 3, "Value of the bit");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: rscmw_GetStatusRegisterBit
 * Purpose:  This function contains the commands for the queries status bit
 *           reporting system.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetStatusRegisterBit(ViSession instrSession,
                                             ViInt32   registerOperation,
                                             ViInt32   registerPart,
                                             ViInt32   bitNumber,
                                             ViBoolean *bitValue)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE];
	ViInt32  i = 0;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerOperation, RSCMW_VAL_QUEST_STAT_REG, RSCMW_VAL_OPER_STAT_REG),
			2, "Register Operation");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, RSCMW_VAL_REG_PART_COND, RSCMW_VAL_REG_PART_EVENT),
			3, "Register Part");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, bitNumber, 8, 12),
			4, "bit Number");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", statusRegisterOperArr[registerOperation], statusRegisterBitsArr[bitNumber - 8],
	         statusRegisterPartQSimpleArr[registerPart]);

	viCheckParm(rscmw_GetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_STATUS_OQ_REGISTER_BIT_CONFIGURATION, &i),
			3, "Value of the bit");

	*bitValue = (i != 0);

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: rscmw_GetStatusErrorQueueEntry
 * Purpose:  This function queries and at the same time deletes the oldest
 *           entry in the error queue. The entry consists of an error number
 *           and a short description of the error. Positive error numbers are
 *           instrument-dependent. Negative error numbers are reserved by the
 *           SCPI standard.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetStatusErrorQueueEntry(ViSession instrSession,
                                                 ViInt32   *errorCode,
                                                 ViChar    errorMessage[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_STATUS_ERROR,
			RS_MAX_MESSAGE_BUF_SIZE, response),
			2, "Error");

	if (sscanf(response, "%ld,\"%256[^\"]", errorCode, errorMessage) != 2)
		checkErr(RS_ERROR_UNEXPECTED_RESPONSE);

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Creates a macro, defined by an identifier string '<MacroLabel>' and
/// HIFN      the macro contents <MacroSequence>. If the macro label exists already,
/// HIFN      the macro contents are overwritten. Macro execution must be enabled
/// HIFN      using *EMC ON. Macros are deleted when a remote connection is closed
/// HIFN      but can be saved to a macro file (MMEMory:STORe:MACRo) for later
/// HIFN      re-use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR macroLabel/String parameter, used to execute the created macro and to reference
/// HIFN      it in other macro commands.
/// HIPAR macroSequence/Block data element defining the macro contents, typically a sequence
/// HIFN      of remote control commands. #0 introduces a data block of indefinite
/// HIFN      length.
ViStatus _VI_FUNC rscmw_CreateMacro(ViSession instrSession,
                                    ViString  macroLabel,
                                    ViString  macroSequence)
{
	ViStatus error = VI_SUCCESS;
	ViChar   *pbuffer = NULL;

	checkErr(RsCore_LockSession(instrSession));

	pbuffer = (ViChar*)malloc(RS_MAX_MESSAGE_BUF_SIZE + strlen(macroSequence));
	sprintf(pbuffer, "*DMC '%s','%s'", macroLabel, macroSequence);
	checkErr(RsCore_Write(instrSession, pbuffer));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	if (pbuffer) free(pbuffer);
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Starts recording of submitted commands into the specified macro file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR macroFile/String parameter specifying path and filename of the source file (on
/// HIFN      the instrument).
ViStatus _VI_FUNC rscmw_StartMacroRecording(ViSession instrSession,
                                            ViString  macroFile)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:REC:MACR:FILE:STAR '%s'", macroFile);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Stops recording of commands into a macro file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_StopMacroRecording(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_Write(instrSession, "SYST:REC:MACR:FILE:STOP"));
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Deletes a macro identified by its 'Macro Label'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR macroLabel/String parameter, used to execute the created macro and to reference
/// HIFN      it in other macro commands.
ViStatus _VI_FUNC rscmw_DeleteMacro(ViSession instrSession,
                                    ViString  macroLabel)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "*RMC '%s'\n", macroLabel);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Deletes all macros of the active remote connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_DeleteAllMacros(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_DELETE_ALL_MACROS, NULL),
			2, "Delete All Macros");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Stores the contents of a macro identified by its <MacroLabel> to a
/// HIFN      file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR macroLabel/String parameter, used to execute the created macro and to reference
/// HIFN      it in other macro commands.
/// HIPAR macroFile/String parameter specifying path and filename of the source file (on
/// HIFN      the instrument).
ViStatus _VI_FUNC rscmw_SaveMacro(ViSession instrSession,
                                  ViString  macroLabel,
                                  ViString  macroFile)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:MACR '%s','%s'\n", macroLabel, macroFile);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Loads a macro file, assigning an identifier 'Macro Label' to the macro
/// HIFN      file contents. Macros are deleted when the remote connection is closed.
/// HIFN      Macro files can be created e.g. using the MMEMory:STORe:MACRo command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR macroLabel/String parameter, used to execute the created macro and to reference
/// HIFN      it in other macro commands.
/// HIPAR macroFile/String parameter specifying path and filename of the source file (on
/// HIFN      the instrument).
ViStatus _VI_FUNC rscmw_LoadMacro(ViSession instrSession,
                                  ViString  macroLabel,
                                  ViString  macroFile)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:MACR '%s','%s'", macroLabel, macroFile);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns the contents of a macro identified by its 'Macro Label'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR macroLabel/String parameter, used to execute the created macro and to reference
/// HIFN      it in other macro commands.
/// HIPAR macroSequence/Block data element containing the macro contents.
/// HIPAR arraySize/Defines size of array 'Macro Sequence'.
ViStatus _VI_FUNC rscmw_QueryMacro(ViSession instrSession,
                                   ViString  macroLabel,
                                   ViUInt32  arraySize,
                                   ViChar    macroSequence[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "*GMC? '%s'", macroLabel);
	checkErr(rscmw_QueryViString(instrSession, cmd, arraySize, macroSequence));

Error:
	return error;
}

/// HIFN  Returns the macro labels of all macros of the active connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR macroLabels/Block data element containing the macro contents. Comma-separated list
/// HIFN      of string parameters.
/// HIPAR arraySize/Defines size of array 'Macro Labels'.
ViStatus _VI_FUNC rscmw_QueryMacroLabels(ViSession instrSession,
                                         ViUInt32  arraySize,
                                         ViChar    macroLabels[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(rscmw_QueryViString(instrSession, "*LMC?", arraySize, macroLabels));

Error:
	return error;
}

/*****************************************************************************
 * Function: rscmw_ConfigureBaseTrigger
 * Purpose:  Configures trigger subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureBaseTrigger(ViSession instrSession,
                                             ViInt32   connector,
                                             ViChar    trigger_source[],
                                             ViInt32   direction)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_INSTR_CON_A, RSCMW_VAL_INSTR_CON_B),
			2, "Connector");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", statusConnectorArr[connector]);

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_SET_BASE_TRIGGER_SOURCE_DIRECTION, direction),
			4, "Set Base Trigger Source Direction");

	if (trigger_source != NULL)
	{
		viCheckParm(rscmw_SetAttributeViString(instrSession, statusConnectorArr[connector], RSCMW_ATTR_SET_BASE_TRIGGER_SOURCE, trigger_source),
				3, "Set Base Trigger Source");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures whether the rising edge or the falling edge of the trigger
/// HIFN  pulse shall be generated at the trigger event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the prefix_init or
/// HIPAR instrSession/prefix_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/Selects the output trigger signals to be routed to the TRIG A and TRIG
/// HIPAR connector/B connectors.
/// HIPAR slope/Specifies whether the rising edge or the falling edge of the trigger
/// HIPAR slope/pulse shall be generated at the trigger event.
ViStatus _VI_FUNC rscmw_ConfigureBaseTriggerSlope(ViSession instrSession,
                                                  ViInt32   connector,
                                                  ViInt32   slope)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_INSTR_CON_A, RSCMW_VAL_INSTR_CON_B),
			2, "Connector");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", statusConnectorArr[connector]);

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_SET_BASE_TRIGGER_SLOPE, slope),
			3, "slope");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Lists all trigger source values that can be set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR connector/Selects the output trigger signals to be routed to the TRIG A and TRIG B connectors.
/// HIPAR optionList/Returns all trigger source values that can be set.
ViStatus _VI_FUNC rscmw_QueryBaseTriggerSourceCatalog(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViInt32   connector,
                                                      ViChar    optionList[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_INSTR_CON_A, RSCMW_VAL_INSTR_CON_B),
			2, "Connector");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", statusConnectorArr[connector]);

	viCheckParm(rscmw_GetAttributeViString(instrSession, repCap, RSCMW_ATTR_BASE_TRIGGER_SOURCE_CATALOG,
			bufferSize, optionList),
			2, "Option List");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Iitiates the generation of a "User Initiated Trigger" signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the prefix_init or
/// HIPAR instrSession/prefix_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userTrigger/Selects the user trigger.
ViStatus _VI_FUNC rscmw_InitiateUserTrigger(ViSession instrSession,
                                            ViInt32   userTrigger)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, userTrigger, 1, 4),
			2, "User Trigger");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "UserTrig%ld", userTrigger);

	checkErr(rscmw_SetAttributeViString(instrSession, repCap, RSCMW_ATTR_INITIATE_USER_TRIGGER, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Query the stored calibration information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR result/Query the stored calibration information.
ViStatus _VI_FUNC rscmw_QueryAllCalibrations(ViSession instrSession,
                                             ViInt32   bufferSize,
                                             ViChar    result[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_CALIBRATION_ALL,
			bufferSize, result),
			2, "Result");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Query the stored information about the latest calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR type/Type of the calibration.
/// HIPAR result/Query the stored calibration information.
ViStatus _VI_FUNC rscmw_QueryLatestCalibration(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViInt32   type,
                                               ViChar    result[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViString typeArr[] = {"", "FSC", "UCOR", "CAL", "OGC"};

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CAL:BASE:LAT? %s\n", typeArr[type]);
	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, result, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Query date and time of the latest calibration of the specified type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR type/Type of the calibration.
/// HIPAR result/Query the name and creation date of the currently active RF path correction file.
ViStatus _VI_FUNC rscmw_QueryCalibrationDateTime(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViInt32   type,
                                                 ViChar    result[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViString typeArr[] = {"", "FSC", "UCOR", "CAL", "OGC"};

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CAL:BASE:LAT:SPEC? %s", typeArr[type]);
	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, result, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Query name and creation date of the currently active RF path correction file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR result/Query the name and creation date of the currently active RF path correction file.
ViStatus _VI_FUNC rscmw_QueryRFCorrectionFile(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViChar    result[])
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_RF_CORRECTION_FILE,
			bufferSize, result),
			2, "Result");

Error:
	return error;
}

/// HIFN  Runs selected operation with buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR operation/Selects an operation with buffer.
/// HIPAR bufferLabel/String parameter, used to identify the buffer in all buffer commands.
ViStatus _VI_FUNC rscmw_BufferOperations(ViSession instrSession,
                                         ViInt32   operation,
                                         ViChar    bufferLabel[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, 0, 4),
			2, "Operation");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, bufferLabel), 3, "Buffer Label");

	switch (operation)
	{
	case RSCMW_VAL_BUFF_START:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_START_BUFFER, bufferLabel),
				3, "Buffer Label");
		break;
	case RSCMW_VAL_BUFF_STOP:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_STOP_BUFFER, bufferLabel),
				3, "Buffer Label");
		break;
	case RSCMW_VAL_BUFF_CLEAR:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CLEAR_BUFFER, bufferLabel),
				3, "Buffer Label");
		break;
	case RSCMW_VAL_BUFF_DELETE:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_DELETE_BUFFER, bufferLabel),
				3, "Buffer Label");
		break;
	case RSCMW_VAL_BUFF_CONTINUE:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CONTINUE_BUFFER, bufferLabel),
				3, "Buffer Label");
		break;
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns the contents of a buffer line no. <Line Number>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR bufferLabel/String parameter, used to identify the buffer in all buffer commands.
/// HIPAR lineNumber/Line of results in the buffer..
/// HIPAR bufferContent/Returns the contents of a buffer line no. <Line Number>.
ViStatus _VI_FUNC rscmw_FetchBuffer(ViSession instrSession,
                                    ViInt32   bufferSize,
                                    ViChar    bufferLabel[],
                                    ViInt32   lineNumber,
                                    ViChar    bufferContent[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BASE:BUFF? '%s',%ld\n", bufferLabel, lineNumber);
	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, bufferContent, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns the size (number of lines) of a buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferLabel/String parameter, used to identify the buffer in all buffer commands.
/// HIPAR numberOfLines/Returns the size (number of lines) of a buffer.
ViStatus _VI_FUNC rscmw_FetchBufferLines(ViSession instrSession,
                                         ViChar    bufferLabel[],
                                         ViInt32   *numberOfLines)
{
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BASE:BUFF:LIN? '%s'\n", bufferLabel);
	checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfLines));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function sets a timestamp with the current system time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
ViStatus _VI_FUNC rscmw_SetTimestamp(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_SET_SYSTEM_TIMESTAMP, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function returns a timestamp with the current system time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR timestamp/Returns timestamp, seven values, separated by commas: year, month, day, hour, minutes, seconds, milliseconds.
ViStatus _VI_FUNC rscmw_QueryTimestamp(ViSession instrSession,
                                       ViInt32   bufferSize,
                                       ViChar    timestamp[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_GET_SYSTEM_TIMESTAMP,
			bufferSize, timestamp),
			2, "Timestamp");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function starts a timer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR timer/Selects a type of timer.
/// HIPAR timeout/Sets the timeout for timer.
ViStatus _VI_FUNC rscmw_StartTimer(ViSession instrSession,
                                   ViInt32   timer,
                                   ViInt32   timeout)
{
	ViStatus error = VI_SUCCESS;
	ViUInt32 timeoutOld = 0;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, timer, 0, 1),
			2, "Timer");

	checkErr(rscmw_GetOPCTimeout(instrSession, &timeoutOld));
	checkErr(rscmw_SetOPCTimeout(instrSession, 2*timeout));

	switch (timer)
	{
	case RSCMW_VAL_TIM_ABS:
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_SET_ABSOLUTE_TIMER, timeout),
				3, "Timeout");
		break;
	case RSCMW_VAL_TIM_REL:
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_SET_RELATIVE_TIMER, timeout),
				3, "Timeout");
		break;
	}

Error:
	checkErr(rscmw_SetOPCTimeout(instrSession, timeoutOld));
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the internal IPv4 subnet of the instrument. Changing the
/// HIFN  <ID> or the <NetworkSegment> initiates a reboot of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nodeID/Selects the node ID, used as third octet of the IPv4 address. N1 to
/// HIPAR nodeID/N254 results in the IP address x.x.1.x to x.x.254.x. Assign different
/// HIPAR nodeID/IDs to all subnet nodes.
/// HIPAR description/String for easy identification of the subnet node, for example in
/// HIPAR description/thesubnet monitor output.
/// HIPAR networkSegment/Selects the network segment to be used for the subnet.
ViStatus _VI_FUNC rscmw_ConfigureSubnetNode(ViSession instrSession,
                                            ViInt32   nodeID,
                                            ViString  description,
                                            ViInt32   networkSegment)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, nodeID, 1, 254),
			2, "Node ID");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, networkSegment, RSCMW_VAL_NETWORK_SEGMENT_A, RSCMW_VAL_NETWORK_SEGMENT_C),
			4, "Network Segment");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:IPS:SNOD N%ld,'%s',%s", nodeID, description, networkSegmentArr[networkSegment]);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Queries the subnet node information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nodeName/Queries the subnet node name of the R&S CMW500.
/// HIPAR nodeType/Queries the subnet node type of the R&S CMW500.
/// HIPAR selectedNetworkSegment/Returns the selected network segment.
/// HIPAR ipAddress/The first two IP address octets for the subnet.
/// HIPAR subnetMask/The used subnet mask (fixed value).
ViStatus _VI_FUNC rscmw_QuerySubnetNodeInfo(ViSession instrSession,
                                            ViChar    nodeName[],
                                            ViChar    nodeType[],
                                            ViInt32   *selectedNetworkSegment,
                                            ViChar    ipAddress[],
                                            ViChar    subnetMask[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViChar   *p2buf;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SUBNET_NODE_NAME,
			RS_MAX_MESSAGE_BUF_SIZE, nodeName),
			2, "Node Name");

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SUBNET_NODE_TYPE,
			RS_MAX_MESSAGE_BUF_SIZE, nodeType),
			3, "Node Type");

	checkErr(rscmw_QueryViString(instrSession, "SENS:BASE:IPS:SNOD:NSEG?", RS_MAX_MESSAGE_BUF_SIZE, response));

	p2buf = strtok(response, ",");

	*selectedNetworkSegment = RsCore_FindStringIndex(networkSegmentArr, p2buf);

	p2buf = strtok(NULL, ",");

	strcpy(ipAddress, p2buf);

	p2buf = strtok(NULL, ",");

	strcpy(subnetMask, p2buf);

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function assigns a subnet conform IP address to a network adapter
/// HIFN  of the instrument, selected via index <n>. As a consequence, DHCP is
/// HIFN  disabled, any IP addresses already assigned to the "Lan Remote" network
/// HIFN  adapter are removed and the displayed fixed IP address is assigned
/// HIFN  instead.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR index/Selects the index of the network adapter.
/// HIPAR setSubnetConform/To assign a subnet conform IP address, send 1 or ON. To try again,
/// HIPAR setSubnetConform/send first 0 or OFF, then again 1 or ON.
ViStatus _VI_FUNC rscmw_ConfigureNetworkAdapter(ViSession instrSession,
                                                ViInt32   index,
                                                ViBoolean setSubnetConform)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 5),
			2, "Index");

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "NWAdapter%ld", index);

	viCheckParm(rscmw_SetAttributeViBoolean(instrSession, repCap, RSCMW_ATTR_NETWORK_ADAPTER, setSubnetConform),
			3, "Set Subnet Conform");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns information about this network adapter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR index/Selects the index of the network adapter.
/// HIPAR networkAdapterName/Returns the name of the network adapter
/// HIPAR IPAddress/String containing the IP address (to be) assigned.
/// HIPAR status/Returns the state indicating whether the returned IP address has been successfully assigned to the network adapter.
ViStatus _VI_FUNC rscmw_QueryNetworkAdapter(ViSession instrSession,
                                            ViInt32   index,
                                            ViChar    networkAdapterName[],
                                            ViChar    IPAddress[],
                                            ViInt32   *status)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViChar   *p2buf;

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:IPS:NWAD%ld?", index);
	checkErr(RsCore_QueryViString(instrSession, cmd, response));

	p2buf = strtok(response, ",");
	strcpy(networkAdapterName, p2buf);
	p2buf = strtok(NULL, ",");
	strcpy(IPAddress, p2buf);
	p2buf = strtok(NULL, ",");

	*status = RsCore_FindStringIndex(statusArr, p2buf);

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Queries the subnet monitor information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nodeNames/Queries the name of all network nodes detected by the subnet monitor.
/// HIPAR nodeTypes/Queries the type of all network nodes detected by the subnet monitor.
/// HIPAR id/Queries the ID (third segment of IP address) of all network nodes
/// HIPAR id/detected by the subnet monitor.
/// HIPAR description/Queries the description of all network nodes detected by the subnet
/// HIPAR description/monitor.
ViStatus _VI_FUNC rscmw_QuerySubnetMonitor(ViSession instrSession,
                                           ViChar    nodeNames[],
                                           ViChar    nodeTypes[],
                                           ViChar    id[],
                                           ViChar    description[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SUBNET_MONITOR_NAME,
			RS_MAX_MESSAGE_BUF_SIZE, nodeNames),
			2, "Node Names");

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SUBNET_MONITOR_TYPE,
			RS_MAX_MESSAGE_BUF_SIZE, nodeTypes),
			3, "Node Types");

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SUBNET_MONITOR_ID,
			RS_MAX_MESSAGE_BUF_SIZE, id),
			4, "ID");

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SUBNET_MONITOR_DESCRIPTION,
			RS_MAX_MESSAGE_BUF_SIZE, description),
			5, "Description");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Initiates an update of the information provided by the subnet monitor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_RefreshSubnetMonitor(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_SUBNET_MONITOR_REFRESH, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Enable or disable tracing of the remote interface to a file for the
/// HIFN      specified subinstrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR subinstrument/Selects the node ID, used as third octet of the IPv4 address. N1 to
/// HIFN      N254 results in the IP address x.x.1.x to x.x.254.x. Assign different
/// HIFN      IDs to all subnet nodes.
/// HIPAR enabled/Enable or disable tracing of the remote interface to a file for the
/// HIFN      specified subinstrument.
ViStatus _VI_FUNC rscmw_ConfigureRemoteTrace(ViSession instrSession,
                                             ViInt32   subinstrument,
                                             ViBoolean enabled)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld", subinstrument);

	viCheckParm(rscmw_SetAttributeViBoolean(instrSession, repCap, RSCMW_ATTR_REMOTE_TRACE_ENABLED, enabled),
			3, "Enabled");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN Enables or disables the display of the SCPI remote trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function.
/// HIPAR mode/Enables or disables the display of the SCPI remote trace.
ViStatus _VI_FUNC rscmw_ConfigureReportDisplay(ViSession instrSession,
                                               ViInt32   mode)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "",
        RSCMW_ATTR_CONFIGURE_REPORT_DISPLAY, mode), 2, "Mode")

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures file name, format and size of remote trace file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR subinstrument/Selects the node ID, used as third octet of the IPv4 address. N1 to
/// HIFN      N254 results in the IP address x.x.1.x to x.x.254.x. Assign different
/// HIFN      IDs to all subnet nodes.
/// HIPAR fileName/Specify path and name of the target file for tracing of the remote
/// HIFN      interface. For different subinstruments, specify different files.
/// HIPAR fileSize/Specifies the maximum size of the trace file in bytes.
/// HIPAR format/Specifies the format of the target file for tracing of the remote
/// HIFN      interface for the specified subinstrument. The trace can be stored as
/// HIFN      ASCII file or as XML file.
ViStatus _VI_FUNC rscmw_ConfigureRemoteTraceFile(ViSession instrSession,
                                                 ViInt32   subinstrument,
                                                 ViString  fileName,
                                                 ViInt32   fileSize,
                                                 ViInt32   format)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld", subinstrument);

	viCheckParm(rscmw_SetAttributeViString(instrSession, repCap, RSCMW_ATTR_REMOTE_TRACE_FILE_PATH, fileName),
			3, "File Name");

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_REMOTE_TRACE_FILE_SIZE, fileSize),
			4, "File Size");

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_REMOTE_TRACE_FORMAT, format),
			5, "Format");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Specifies a filter for tracing of the specified subinstrument. The
/// HIFN      filter defines which message types and events are traced into a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR subinstrument/Selects the node ID, used as third octet of the IPv4 address. N1 to
/// HIFN      N254 results in the IP address x.x.1.x to x.x.254.x. Assign different
/// HIFN      IDs to all subnet nodes.
/// HIPAR arraySize/Defines size of array 'States'.
/// HIPAR states/Specifies a filter for tracing of the specified subinstrument. The
/// HIFN      filter defines which message types and events are traced into a file.
/// HIFN      Currently 8 types and events are supported.
ViStatus _VI_FUNC rscmw_ConfigureRemoteTraceFilter(ViSession instrSession,
                                                   ViInt32   subinstrument,
                                                   ViInt32   arraySize,
                                                   ViBoolean states[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViInt32  i = 0;
	ViInt32  filtersCountFromUserManual = 8;

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:REM:MODE:FILE%ld:FILT ", subinstrument);

	for (i = 0; i < filtersCountFromUserManual; i++)
	{
		if (i < arraySize)
			strcat(cmd, states[i] ? "ON" : "OFF");
		else
			strcat(cmd, "OFF");
		if (i < (filtersCountFromUserManual - 1))
			strcat(cmd, ",");
	}

	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Specifies how / when tracing is started and stopped.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIFN      function to select the desired instrument driver session.
/// HIPAR subinstrument/Selects the node ID, used as third octet of the IPv4 address. N1 to
/// HIFN      N254 results in the IP address x.x.1.x to x.x.254.x. Assign different
/// HIFN      IDs to all subnet nodes.
/// HIPAR startMode/Specifies whether tracing is started automatically or manually.
/// HIPAR stopMode/Specifies how / when tracing is stopped and the trace file is closed.
ViStatus _VI_FUNC rscmw_ConfigureRemoteTraceStartStopMode(ViSession instrSession,
                                                          ViInt32   subinstrument,
                                                          ViInt32   startMode,
                                                          ViInt32   stopMode)
{
	ViStatus error = VI_SUCCESS;
	ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld", subinstrument);

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_REMOTE_TRACE_START_MODE, startMode),
			3, "Start Mode");

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_REMOTE_TRACE_STOP_MODE, stopMode),
			4, "Stop Mode");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function clears the display of the SCPI remote trace in analysis
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rscmw_ClearSCPIRemoteTraceDisplay(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_CLEAR_REMOTE_TRACE_DISPLAY, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_ConfigureAttenuationCorrectionTable
 * Purpose:  This function contains the commands for the configuring frequency
 *           depended attenuation table.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureAttenuationCorrectionTable(ViSession instrSession,
                                                            ViInt32   tableOperation,
                                                            ViString  tableName,
                                                            ViString  tableData)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, tableName), 3, "Table Name");

	switch (tableOperation)
	{
	case RSCMW_VAL_ATT_TAB_CRE:
		if (tableData == NULL)
			snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:FDC:CTAB:CRE '%s'\n", tableName);
		else
			snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:FDC:CTAB:CRE '%s',%s\n", tableName, tableData);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));
		break;
	case RSCMW_VAL_ATT_TAB_ADD:
		if (tableData == NULL)
			snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:FDC:CTAB:ADD '%s'\n", tableName);
		else
			snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:FDC:CTAB:ADD '%s',%s\n", tableName, tableData);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));

		break;
	case RSCMW_VAL_ATT_TAB_DEL:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_DELETE_ATTENUATION_CORRECTION_TABLE, tableName),
				2, "Delete Attenuation Correction Table");
		break;
	case RSCMW_VAL_ATT_TAB_ERASE:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:FDC:CTAB:ERAS '%s',%s\n", tableName, tableData);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));
		break;
	case RSCMW_VAL_ATT_TAB_DEL_ALL:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_DELETE_ALL_ATTENUATION_CORRECTION_TABLE, NULL));
		break;
	case RSCMW_VAL_ATT_TAB_SAVE:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_SAVE_ATTENUATION_CORRECTION_TABLE, NULL));
		break;
	case RSCMW_VAL_ATT_TAB_LOAD:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_LOAD_ATTENUATION_CORRECTION_TABLE, NULL));
		break;
	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, tableOperation), 2, "Table Operation");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_ConfigureAttenuationCorrectionTableDirection
 * Purpose:  This function activates / deactives a correction table for the
 *           signal path using a specific RF connector.
 *
 *           For bidirectional connectors the table can be applied to both
 *           directions or to one direction. It is possible to assign different
 *           tables to the directions of a bidirectional connector, see example
 *           in documentation.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ConfigureAttenuationCorrectionTableDirection(
							ViSession instrSession,
							ViInt32   tableOperation,
							ViInt32   connector,
							ViString  tableName,
							ViInt32   direction,
							ViInt32   rfConverter)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, tableOperation, RSCMW_VAL_ATT_TAB_ACT, RSCMW_VAL_ATT_TAB_DEACT),
			2, "Table Operation");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_CON_RF1OUT, RSCMW_VAL_CON_RFBCOM),
			3, "Connector");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, rfConverter, RSCMW_VAL_CONVERTER_RF1, RSCMW_VAL_CONVERTER_RF4),
			6, "RF Converter");

	switch (tableOperation)
	{
	case RSCMW_VAL_ATT_TAB_ACT:
		viCheckParm(RsCore_InvalidNullPointer(instrSession, tableName), 4, "Table Name");
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:FDC:ACT %s,'%s',%s,%s\n", attConnectorCMDArr[connector], tableName,
		         attDirectionArr[direction], rfConverterArr[rfConverter - 1]);
		checkErr(RsCore_Write(instrSession, cmd));
		break;
	case RSCMW_VAL_ATT_TAB_DEACT:
		/*
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_DEACTIVATE_CORRECTION_TABLE_FOR_CONNECTOR, connector),
				3, "Deactivate Correction Table For Connector");
		*/
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:FDC:DEAC %s,%s,%s\n", attConnectorCMDArr[connector], attDirectionArr[direction], rfConverterArr[rfConverter - 1]);
		checkErr(RsCore_Write(instrSession, cmd));
		break;
	}

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Deactivates all correction tables for all RF connectors of a selected subinstrument.
/// HIFN  For bidirectional connectors, the tables can be deactivated for both directions
/// HIFN  or for one direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR direction/This control specifies the direction to which the correction table
/// HIPAR direction/shall be applied. RX means input and TX means output. For a pure output
/// HIPAR direction/connector RX is ignored.
/// HIPAR tablePath/String selecting the subinstrument.
ViStatus _VI_FUNC rscmw_DeactivateAllAttenuationCorrectionTables(
							ViSession instrSession,
							ViInt32   direction,
							ViString  tablePath)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMW_VAL_DIRECTION_RX, RSCMW_VAL_DIRECTION_RT),
			2, "Direction");

	if (tablePath != NULL)
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:FDC:DEAC:ALL %s,'%s'", attDirectionArr[direction], tablePath);
	else
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:FDC:DEAC:ALL %s", attDirectionArr[direction]);
	checkErr(RsCore_Write(instrSession, cmd));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Queries whether a correction table with the specified name exists or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR tableName/This control specifies the table in which the command will be performed.
/// HIPAR result/Returns whether a correction table with the specified name exists or not.
ViStatus _VI_FUNC rscmw_QueryAttenuationCorrectionTableExistance(
							ViSession instrSession,
							ViString  tableName,
							ViInt32   *result)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, tableName), 2, "Table Name");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONFigure:BASE:FDCorrection:CTABle:EXISt? '%s'", tableName);
	checkErr(rscmw_QueryViInt32(instrSession, cmd, result));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function activates / deactives a correction table for the signal
/// HIFN  path using a specific RF connector.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function/commands are not compatible with R&S CMW500.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR tableOperation/This control selects the table operation.
/// HIPAR direction/Selects the direction.
/// HIPAR connector/Selects a single connector or a bench with 4 or 8 connectors
/// HIPAR tables/String parameter identifying the table for the selected single
/// HIPAR tables/connector or for the first up to eigth connector of the selected bench
ViStatus _VI_FUNC rscmw_ConfigureCMWSAttenuationCorrectionTableDirection(
							ViSession instrSession,
							ViInt32   tableOperation,
							ViInt32   direction,
							ViInt32   connector,
							ViString  tables)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
	ViChar   repCap[RS_REPCAP_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMW_VAL_INPUT_RX, RSCMW_VAL_OUTPUT_TX),
			3, "Direction");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_CON_R11, RSCMW_VAL_CON_RB14),
			4, "Connector");

	switch (tableOperation)
	{
	case RSCMW_VAL_ATT_TAB_ACT:
		viCheckParm(RsCore_InvalidNullPointer(instrSession, tables), 4, "Table Name");

		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:CMWS:FDC:ACT:%s %s,%s\n", attDirectionCMWSArr[direction], attConnectorCMWSArr[connector], tables);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));
		break;
	case RSCMW_VAL_ATT_TAB_DEACT:
		snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", attDirectionCMWSArr[direction]);

		viCheckParm(rscmw_SetAttributeViInt32(instrSession, repCap, RSCMW_ATTR_DEACTIVATE_CORRECTION_TABLE_FOR_CONNECTOR_CMWS, connector),
				4, "Deactivate Correction Table For Connector");
		break;
	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, tableOperation), 2, "Table Operation");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function activates or deactivates a single RF connector. When an
/// HIFN  output signal from the R&S CMW500 is fed into the R&S CMWS, it is
/// HIFN  available at the active RF connectors of the connected bench.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function/commands are not compatible with R&S CMW500.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR connector/Selects a single connector.
/// HIPAR connectorState/Enables or disables selected connector.
ViStatus _VI_FUNC rscmw_ConfigureCMWSGPRFGeneratorSingleRFConnector(
							ViSession instrSession,
							ViInt32   instance,
							ViInt32   connector,
							ViBoolean connectorState)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_CON_R11, RSCMW_VAL_CON_RB4),
			3, "Connector");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:GPRF:GEN%ld:CMWS:USAG:TX %s,%s\n", instance, attConnectorCMWSArr[connector],
	         offOnArr[connectorState]);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function activates or deactivates the individual connectors of a
/// HIFN  bench. When an output signal from the R&S CMW500 is fed into the R&S
/// HIFN  CMWS, it is available at the active RF connectors of the connected
/// HIFN  bench.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function/commands are not compatible with R&S CMW500.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR connector/Selects a bench with 4 or 8 connectors
/// HIPAR bufferSize/Pass the number of bytes in the ViBoolean array you specify for the
/// HIPAR bufferSize/Connector State parameter.
/// HIPAR connectorState/Comma separated list of 4 or 8 values, one for each connector of
ViStatus _VI_FUNC rscmw_ConfigureCMWSGPRFGeneratorIndividualRFConnector(
							ViSession instrSession,
							ViInt32   instance,
							ViInt32   connector,
							ViInt32   bufferSize,
							ViBoolean connectorState[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

	instance;
	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_CON_R118, RSCMW_VAL_CON_RB14),
			3, "Connector");

	checkErr(RsCore_WriteAsciiViBooleanArray(instrSession, cmd, connectorState, bufferSize));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Deactivates all correction tables for all RF connectors of a selected subinstrument.
/// HIFN  For bidirectional connectors, the tables can be deactivated for both directions
/// HIFN  or for one direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR direction/Selects the direction.
/// HIPAR tablePath/String selecting the subinstrument.
ViStatus _VI_FUNC rscmw_DeactivateCMWSAllAttenuationCorrectionTables(
							ViSession instrSession,
							ViInt32   direction,
							ViString  tablePath)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMW_VAL_INPUT_RX, RSCMW_VAL_IO_ALL),
			2, "Direction");

	if (tablePath != NULL)
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:CMWS:FDC:DEAC:%s '%s'\n", attDirectionTablesArr[direction], tablePath);
	else
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:CMWS:FDC:DEAC:%s\n", attDirectionTablesArr[direction]);
	checkErr(RsCore_Write(instrSession, cmd));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_GetListOfAttenuationTables
 * Purpose:  This function returns a comma separated list of table names. All
 *           correction tables currently stored on the hard disk are listed.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetListOfAttenuationTables(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    output[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, output), 5, "Output");

	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CONF:BASE:FDC:CTAB:CAT?\n", bufferSize, output, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  on the hard disk for the addressed sub-instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfTables/Returns the number of correction tables currently stored on the hard
/// HIPAR numberOfTables/disk for the addressed sub-instrument..
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTablesCount(ViSession instrSession,
                                                            ViInt32   *numberOfTables)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViInt32(instrSession, "", RSCMW_ATTR_ATTENUATION_CORRECTION_TABLE_COUNT, numberOfTables),
			2, "Number Of Tables");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_GetAttenuationCorrectionTable
 * Purpose:  This function lists the correction tables assigned to a specific
 *           RF connector.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTable(ViSession instrSession,
                                                      ViInt32   connector,
                                                      ViInt32   rfConverter,
                                                      ViInt32   bufferSize,
                                                      ViChar    assignedTables[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_CON_RF1OUT, RSCMW_VAL_CON_RF4COM),
			2, "Connector");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, rfConverter, RSCMW_VAL_CONVERTER_RF1, RSCMW_VAL_CONVERTER_RF4),
			3, "RF Converter");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, assignedTables), 5, "Assigned Tables");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:FDC:USAG? %s,%s\n", attConnectorCMDArr[connector], rfConverterArr[rfConverter - 1]);
	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, assignedTables, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function lists the correction tables assigned to a selected RF
/// HIFN  connector of the R&S CMWS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR connector/Selects a single connector
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR assignedTables/Displays strings identifying the tables assigned to the RX or TX
/// HIPAR assignedTables/direction. If no table is active an empty string is returned. For pure
/// HIPAR assignedTables/output connectors the RX string is always empty.
ViStatus _VI_FUNC rscmw_GetCMWSAttenuationCorrectionTable(ViSession instrSession,
                                                          ViInt32   connector,
                                                          ViInt32   bufferSize,
                                                          ViChar    assignedTables[])
{
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, connector, RSCMW_VAL_CON_R11, RSCMW_VAL_CON_RB4),
			2, "Connector");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, assignedTables), 4, "Assigned Tables");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:CMWS:FDC:USAG? %s\n", attConnectorCMWSArr[connector]);
	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, assignedTables, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/****************************************************************************
 * Function: rscmw_GetAttenuationCorrectionTableDetails
 * Purpose:  This function returns the entries of a correction table. Each
 *           entry consists of a frequency [Hz] and an attenuation [dB].
 *           Example(s):
 *           'mytable', 3, 1
 *           Returns the entry with index number 3, e.g.: 2000000000,0.7
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTableDetails(ViSession instrSession,
                                                             ViString  tableName,
                                                             ViInt32   startIndex,
                                                             ViInt32   count,
                                                             ViInt32   bufferSize,
                                                             ViChar    details[])
{
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViStatus error = VI_SUCCESS;
	ViChar   param[RS_MAX_MESSAGE_LEN];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, tableName), 2, "Table Name");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, details), 6, "Details");

	if (startIndex >= 0)
	{
		if (count > 0)
		{
			snprintf(param, RS_MAX_MESSAGE_LEN, "'%s' ,%ld, %ld", tableName, startIndex, count);
		}
		else
		{
			snprintf(param, RS_MAX_MESSAGE_LEN, "'%s' ,%ld", tableName, startIndex);
		}
	}
	else
	{
		snprintf(param, RS_MAX_MESSAGE_LEN, "'%s'", tableName);
	}

	snprintf(cmd, bufferSize, "CONF:BASE:FDC:CTAB:DET? %s", param);
	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, details, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_GetAttenuationCorrectionTableEntries
 * Purpose:  This function returns number of entries (i.e. pairs of frequency
 *           and attenuation) of a correction table.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetAttenuationCorrectionTableEntries(ViSession instrSession,
                                                             ViString  tableName,
                                                             ViInt32   *entries)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	viCheckParm(RsCore_InvalidNullPointer(instrSession, tableName), 2, "Table Name");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:BASE:FDC:CTAB:LENG? '%s'", tableName);
	checkErr(rscmw_QueryViInt32(instrSession, cmd, entries));

Error:
	return error;
}

/*****************************************************************************
 * Function: File Manager Operations
 * Purpose:  The MMEMory (mass memory) subsystem function provides commands
 *           which allow for access to the storage media of the instrument and
 *           for storing and loading various instrument settings.
 *           The File to be Printed/Plotted command stores the HCOPy outputs
 *           in a file. The various drives can be addressed via the mass
 *           storage unit specifier <msus> using the conventional DOS syntax.
 *           The internal hard disk is addressed by "C:", the floppy-disk drive
 *           installed by "A:". The file names <file_name> are indicated as
 *           string parameters with the commands being enclosed in quotation
 *           marks. They correspond to the DOS conventions.
 *           DOS file names consist of max. 8 ASCII characters and an
 *           extension of up to three characters separated from the file name
 *           by a colon "." Both, the colon and the extension are optional. The
 *           colon is not part of the file name. DOS file names do not differ
 *           between uppercase and lowercase notation. All letters and digits
 *           are permitted as well as the special characters "_", "^", "$",
 *           "~", "!", "#", "%", "&", "-", "{","}", "(", ")", "@" and "' ".
 *           Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 to
 *           LPT3, NUL and PRN. The two characters "*" and "?" have the
 *           function of so-called "wildcards", i.e., they are variables for
 *           selection of several files. The question mark "?" replaces
 *           exactly one character which may be any, the asterisk means any of
 *           the remaining characters in the file name. "*.*" thus means all
 *           files in a directory.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_FileManagerOperations(ViSession instrSession,
                                              ViInt32   operation,
                                              ViString  source,
                                              ViString  destination)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, 0, 6),
			2, "Operation");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, source), 3, "Source");

	switch (operation)
	{
	case RSCMW_VAL_FILE_NEW:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_MAKE_DIRECTORY, source));
		break;
	case RSCMW_VAL_FILE_COPY:
		viCheckParm(RsCore_InvalidNullPointer(instrSession, destination), 4, "Destination");
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:COPY '%s','%s'", source, destination);
		checkErr(RsCore_Write(instrSession, cmd));
		break;
	case RSCMW_VAL_FILE_RENAME:
		viCheckParm(RsCore_InvalidNullPointer(instrSession, destination), 4, "Destination");
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:MOVE '%s','%s'", source, destination);
		checkErr(RsCore_Write(instrSession, cmd));
		break;
	case RSCMW_VAL_FILE_DELETE:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_DELETE, source));
		break;
	case RSCMW_VAL_FILE_RDIR:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_DELETE_DIRECTORY, source));
		break;
	case RSCMW_VAL_FILE_CDIR:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_EDIT_PATH, source));
		break;
	case RSCMW_VAL_FILE_CDRIVE:
		checkErr(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_CHANGE_DRIVE, source));
		break;
	}

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Get Current Directory
 * Purpose:  This function returns the default directory for mass memory
 *           storage (current directory).
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetCurrentDirectory(ViSession instrSession,
                                            ViInt32   Buffer_Size,
                                            ViChar    currentDirectory[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_EDIT_PATH,
			Buffer_Size, currentDirectory),
			2, "Current Directory");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN This function returns the current drive.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmw_init
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR currentDrive/Returns the current drive.
ViStatus _VI_FUNC rscmw_GetCurrentDrive(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    currentDrive[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_CHANGE_DRIVE,
			bufferSize, currentDrive),
			2, "Current Drive");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_GetInstrumentDrives
 * Purpose:  This function returns a list of the drives of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetInstrumentDrives(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViChar    instrumentDrives[])
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_GET_INSTRUMENT_DRIVES,
			bufferSize, instrumentDrives),
			2, "File Manager Get Instrument Drives");

Error:
	return error;
}


/*****************************************************************************
 * Function: File Directory Path
 * Purpose:  This fuction reads out the subdirectories or subdirectories and
 *           files in the specified directory.
 ******************************************************************************/
ViStatus _VI_FUNC rscmw_FileDirectoryPath(ViSession instrSession,
                                          ViInt32   read,
                                          ViString  directory,
                                          ViInt32   bufferSize,
                                          ViChar    output[],
                                          ViInt32   *returnedValues)
{
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, output), 5, "Output");

	switch (read)
	{
	case RSCMW_VAL_MMEM_CATALOG_FILES:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:CAT?");
		break;
	case RSCMW_VAL_MMEM_CATALOG_SUBDIR:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:DCAT?");
		break;
	}

	if (directory != NULL)
		RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, " '", directory, "'");

	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, output, returnedValues));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_FileDirectoryPathCount
 * Purpose:  This function returns the number of files and/or subdirectories
 *           of the current or of a specified directory. The number includes
 *           the parent directory strings "." and ".." so that it corresponds
 *           to the number of strings returned by the File Directory Path
 *           function  after the initial numeric parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_FileDirectoryPathCount(ViSession instrSession,
                                               ViInt32   read,
                                               ViString  directory,
                                               ViInt32   *numbers_of_dir)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	switch (read)
	{
	case RSCMW_VAL_MMEM_CATALOG_FILES:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:CAT:LENG?");
		break;
	case RSCMW_VAL_MMEM_CATALOG_SUBDIR:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:DCAT:LENG?");
		break;
	}
	if (directory != NULL)
	{
		RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, " '", directory, "'");
	}

	checkErr(rscmw_QueryViInt32(instrSession, cmd, numbers_of_dir));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function allows to provide data set operation for Save/Recall
/// HIFN  items of memory subsystem.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR stateNumber/Defines save and recall state position.
/// HIPAR path/This control specifies the source of the operation to be performed.
ViStatus _VI_FUNC rscmw_DataSetFileOperations(ViSession instrSession,
                                              ViInt32   operation,
                                              ViInt32   stateNumber,
                                              ViString  path)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	switch (operation)
	{
	case RSCMW_VAL_MEM_SAVE:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "*SAV %ld;MMEM:STOR:STAT %ld,'%s'", stateNumber, stateNumber, path);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));
		break;
	case RSCMW_VAL_MEM_RECALL:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:STAT %ld,'%s';*RCL %ld", stateNumber, path, stateNumber);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));
		break;
	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function executes a partial recall or save, i.e. restores or
/// HIFN  saves the settings of a specific application from the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmw_init or
/// HIPAR instrSession/rscmw_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR itemName/String parameter identifying the application to be saved/recalled.
/// HIPAR path/This control specifies the source/target of the operation to be
/// HIPAR path/performed.
ViStatus _VI_FUNC rscmw_DataSetApplicationOperations(ViSession instrSession,
                                                     ViInt32   operation,
                                                     ViString  itemName,
                                                     ViString  path)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	switch (operation)
	{
	case RSCMW_VAL_MEM_SAVE:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:ITEM '%s','%s'\n", itemName, path);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));
		break;
	case RSCMW_VAL_MEM_RECALL:
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:ITEM '%s','%s'\n", itemName, path);
		checkErr(RsCore_Write(instrSession, cmd));
		checkErr(rscmw_CheckStatus(instrSession));
		break;
	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Sets file attributes for files and directories.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/String parameter to specify the name and/or path of the objects
/// HIPAR fileName/for which the attributes shall be modified.
/// HIPAR action/Attribute actions to be performed.
ViStatus _VI_FUNC rscmw_SetFileAttributes(ViSession instrSession,
                                          ViString  fileName,
                                          ViString  action)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEMory:ATTRibute '%s','%s'", fileName, action);
	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Reads file attributes for files and directories.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR file/String parameter to specify the name and/or path of the objects
/// HIPAR file/for which the attributes shall be modified.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR output/Information strings returned
/// HIPAR returnedValues/Returns the size of string returned from the instrument.
ViStatus _VI_FUNC rscmw_ReadFileAttributes(ViSession instrSession,
                                           ViString  file,
                                           ViInt32   bufferSize,
                                           ViChar    output[],
                                           ViInt32   *returnedValues)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, output), 4, "Output");

	if (file != NULL)
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:ATTR? '%s'", file);
	else
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:ATTR?");

	checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, output, returnedValues));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns the defined alias entries and the assigned directories.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR aliases/Returns a list of the drives of the instrument.
ViStatus _VI_FUNC rscmw_GetAliases(ViSession instrSession,
                                   ViInt32   bufferSize,
                                   ViChar    aliases[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_MANAGER_ALIASES,
			bufferSize, aliases),
			2, "Aliases");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN Specifies the format of screenshots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR format/Specifies the format of screenshots.
ViStatus _VI_FUNC rscmw_ConfigureScreenshotFormat(ViSession instrSession,
                                                  ViInt32   format)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_HARDCOPY_FILE_FORMAT, format),
			2, "Format");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN Specifies the format of screenshots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR area/Selects the part of the screen to be captured.
ViStatus _VI_FUNC rscmw_ConfigureScreenshotArea(ViSession instrSession,
                                                ViInt32   area)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_HARDCOPY_FILE_AREA, area),
			2, "Area");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Captures a screenshot and stores it to the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR location/Selects the window capture type.
/// HIPAR fileName/String parameter specifying the absolute path and name of the file.
ViStatus _VI_FUNC rscmw_SaveScreenshotToFile(ViSession instrSession,
                                             ViInt32   location,
                                             ViString  fileName)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, location, RSCMW_VAL_CAPTURE_WINDOW_ENTIRE, RSCMW_VAL_CAPTURE_WINDOW_INTERIOR),
			2, "Location");

	switch (location)
	{
	case RSCMW_VAL_CAPTURE_WINDOW_ENTIRE:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_CAPTURE_ENTIRE_WINDOW, fileName),
				3, "File Name");
		break;

	case RSCMW_VAL_CAPTURE_WINDOW_INTERIOR:
		viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_FILE_CAPTURE_INTERIOR_WINDOW, fileName),
				3, "File Name");
		break;
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmw_SetAttributeViInt32(ViSession instrSession,
                                            ViString  channelName,
                                            ViUInt32  attributeId,
                                            ViInt32   value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmw_SetAttributeViReal64(ViSession instrSession,
                                             ViString  channelName,
                                             ViUInt32  attributeId,
                                             ViReal64  value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViString */
ViStatus _VI_FUNC rscmw_SetAttributeViString(ViSession instrSession,
                                             ViString  channelName,
                                             ViUInt32  attributeId,
                                             ViString  value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmw_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  channelName,
                                              ViUInt32  attributeId,
                                              ViBoolean value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmw_SetAttributeViSession(ViSession instrSession,
                                              ViString  channelName,
                                              ViUInt32  attributeId,
                                              ViSession value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmw_GetAttributeViInt32(ViSession instrSession,
                                            ViString  channelName,
                                            ViUInt32  attributeId,
                                            ViInt32   *value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);
	error = RsCore_GetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmw_GetAttributeViReal64(ViSession instrSession,
                                             ViString  channelName,
                                             ViUInt32  attributeId,
                                             ViReal64  *value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_GetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViString */
ViStatus _VI_FUNC rscmw_GetAttributeViString(ViSession instrSession,
                                             ViString  channelName,
                                             ViUInt32  attributeId,
                                             ViInt32   bufSize,
                                             ViChar    value[])
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_GetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, bufSize,
	                                    value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmw_GetAttributeViBoolean(ViSession instrSession,
                                              ViString  channelName,
                                              ViUInt32  attributeId,
                                              ViBoolean *value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_GetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmw_GetAttributeViSession(ViSession instrSession,
                                              ViString  channelName,
                                              ViUInt32  attributeId,
                                              ViSession *value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_GetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rscmw_GetAttributeRepCapName(ViSession instrSession,
                                               ViUInt32  attributeId,
                                               ViString  repCapNameId,
                                               ViInt32   index,
                                               ViInt32   bufferSize,
                                               ViChar    repCapName[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViUInt32  attributeID,
                                                          ViInt32   bufferSize,
                                                          ViChar    _VI_FAR repeatedCapabilityIds[])
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
			3, "Buffer Size");

	checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
	return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetAttributeRepeatedCapabilityIdNames(
							ViSession instrSession,
							ViUInt32  attributeID,
							ViString  repeatedCapabilityId,
							ViInt32   bufferSize,
							ViChar    _VI_FAR repeatedCapabilityIdNames[])
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
			4, "Buffer Size");

	checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
	return error;
}

/*****************************************************************************
 * Function: rscmw_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rscmw_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmw_CheckAttributeViInt32(ViSession instrSession,
                                              ViString  channelName,
                                              ViUInt32  attributeId,
                                              ViInt32   value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_CheckAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmw_CheckAttributeViReal64(ViSession instrSession,
                                               ViString  channelName,
                                               ViUInt32  attributeId,
                                               ViReal64  value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_CheckAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViString */
ViStatus _VI_FUNC rscmw_CheckAttributeViString(ViSession instrSession,
                                               ViString  channelName,
                                               ViUInt32  attributeId,
                                               ViString  value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_CheckAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmw_CheckAttributeViBoolean(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViBoolean value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_CheckAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmw_CheckAttributeViSession(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViSession value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_CheckAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_reset(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	if (RsCore_Simulating(instrSession) == VI_FALSE)
	{
		checkErr(rscmw_ClearStatus(instrSession));
		checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
		checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
	}

	checkErr(rscmw_DefaultInstrSetup(instrSession));
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);

	return error;
}

/// HIFN  This function sets them to default values suitable for remote
/// HIFN  operation. The function can be applied to a selected application of the
/// HIFN  sub-instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the prefix_init or
/// HIPAR instrSession/prefix_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR application/String specifying the application to be reset.
ViStatus _VI_FUNC rscmw_resetApplication(ViSession instrSession,
                                         ViString  application)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_SYSTEM_RESET_APPLICATION, application),
			2, "Application");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_self_test(ViSession instrSession,
                                  ViInt16   *testResult,
                                  ViChar    testMessage[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

	checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_revision_query(ViSession instrSession,
                                       ViChar    driverRev[],
                                       ViChar    instrRev[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, driverRev), 2, "Null address for Instrument Driver Revision");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, instrRev), 3, "Null address for Instrument Revision");

	checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
	checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_SystemVersion(ViSession instrSession,
                                      ViInt32   length,
                                      ViChar    systemVersion[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SYSTEM_VERSION,
			length, systemVersion),
			2, "System Version");
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_error_query(ViSession instrSession,
                                    ViInt32   *errCode,
                                    ViChar    errMessage[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, errCode), 2, "Null address for Error Code");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, errMessage), 3, "Null address for Error Message");

	checkErr(RsCore_ErrorQueryAll(instrSession, errCode, 1024, errMessage));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_error_code_query
 * Purpose:  Queries and at the same time deletes the oldest entry in the
 *           error queue.
 *
 *           The command returns the error number without any description of
 *           the error. Positive error numbers are instrument-dependent.
 *           Negative error numbers are reserved by the SCPI standard.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_error_code_query(ViSession instrSession,
                                         ViInt32   *errorCode)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(rscmw_GetAttributeViInt32(instrSession, "", RSCMW_ATTR_SYST_ERR_CODE, errorCode),
			2, "System Error Code");

Error:
	return error;
}

/*****************************************************************************
 * Function: Error List
 * Purpose:  This function queries and at the same time deletes all entries
 *           in the error queue.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ErrorList(ViSession instrSession,
                                  ViInt32   *code,
                                  ViInt32   bufferSize,
                                  ViChar    errors[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 0, 32001),
			2, "Buffer Size");

	viCheckParm(rscmw_GetAttributeViString(instrSession, "", RSCMW_ATTR_SYSTEM_ALL_ERRORS,
			bufferSize, errors),
			4, "Errors message");

	viCheckParm(rscmw_GetAttributeViInt32(instrSession, "", RSCMW_ATTR_SYSTEM_ALL_ERRORS_CODE, code),
			2, "System All Errors Code");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[])
{
	ViStatus error = VI_SUCCESS;
	static   RsCoreStringValueTable errorTable =
	         {
	         RSCMW_ERROR_CODES_AND_MSGS,
	{0,      NULL}
	         };

	(void)(RsCore_LockSession(instrSession));

	// all VISA and RS error codes are handled as well as codes in the table
	viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

	checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:
	(void)RsCore_UnlockSession(instrSession); // IGNORE: ERROR!!! Missing Lock
	return error;
}

/*============================================================================/
 * Function: Store and Load Instrument Data
 * Purpose:  This function stores or recalls instrument data.
/============================================================================*/
ViStatus _VI_FUNC rscmw_storeLoadInstrumentData(ViSession instrSession,
                                                ViInt32   networkMode,
                                                ViInt32   stateNumber)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, networkMode, 0, 1),
			3, "Network mode");

	switch (networkMode)
	{
	case RSCMW_VAL_SAVE:
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_SAV, stateNumber),
				3, "State Number");
		break;

	case RSCMW_VAL_RECALL:
		viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RSCMW_ATTR_RCL, stateNumber),
				3, "State Number");
		break;
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Preset
 * Purpose:  This function initiates an instrument reset.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_Preset(ViSession instrSession,
                               ViInt32   reference)
{
	ViStatus error = VI_SUCCESS;

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, reference, RSCMW_VAL_RST, RSCMW_VAL_RST_ALL),
			2, "Reference");


	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_SetAttributeViString(instrSession, presetPartArr[reference], RSCMW_ATTR_SYSTEM_PRESET, NULL));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function sets the parameters of the current sub-instrument to
/// HIFN  default values suitable for local/manual interaction. The function can
/// HIFN  be applied to a selected application of the sub-instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the prefix_init or
/// HIPAR instrSession/prefix_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR application/String specifying the application to be preset.
ViStatus _VI_FUNC rscmw_PresetApplication(ViSession instrSession,
                                          ViString  application)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViString(instrSession, "", RSCMW_ATTR_SYSTEM_PRESET_APPLICATION, application),
			2, "Application");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  A PRESet sets the parameters to values suitable for local/manual interaction,
/// HIFN  a RESet to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the prefix_init or
/// HIPAR instrSession/prefix_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Part for preset.
ViStatus _VI_FUNC rscmw_ResetDevice(ViSession instrSession,
                                    ViInt32   reference)
{
	ViStatus error = VI_SUCCESS;
	ViAttr   attr[] = {0, RSCMW_ATTR_SYSTEM_RESET_BASE, RSCMW_ATTR_SYSTEM_RESET_ALL};

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, reference, RSCMW_VAL_RST_BASE, RSCMW_VAL_RST_ALL),
			2, "Reference");

	checkErr(rscmw_SetAttributeViString(instrSession, "", attr[reference], NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmw_ClearStatus(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_ClearStatus(instrSession));
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rscmw_IDQueryResponse(ViSession instrSession,
                                        ViChar    IDQueryResponse[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_QueryViString(instrSession, "*IDN?", 256, IDQueryResponse));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmw_ProcessAllPreviousCommands(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_WriteInstrData(instrSession, "*WAI"));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwam_init or
/// HIPAR instrSession/rscmwam_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rscmw_QueryOPC(ViSession instrSession,
                                 ViInt32   *OPC)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(rscmw_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function switches ON/OFF option checking of the instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Document parameter here.
ViStatus _VI_FUNC rscmw_setCheckOption(ViSession instrSession,
                                       ViBoolean state)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	rsSession->optionChecking = state;

Error:
	return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmw_init or
/// HIPAR instrSession/rscmw_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmw_SetVISATimeout(ViSession instrSession,
                                       ViUInt32  VISATimeout)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmw_init or
/// HIPAR instrSession/rscmw_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rscmw_GetVISATimeout(ViSession instrSession,
                                       ViUInt32  *VISATimeout)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rscmw_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	rsSession->optionChecking = optionChecking;
	checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
	checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
	return error;
}

/*****************************************************************************
 * Function: Reliability Indicator
 * Purpose:  This function converts a reliability code returned by an
 *           instrument driver function into a user-readable string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_ReliabilityIndicator(ViSession instrSession,
                                             ViInt32   reliabilityCode,
                                             ViChar    reliabilityMessage[])
{
	ViStatus  error = VI_SUCCESS;
	ViInt16   cnt = 0;
	ViBoolean found = VI_FALSE;

	static RsCoreStringValueTable reliabilityTable =
	{
		{0,   "OK"},
		{1,   "Measurement Timeout"},
		{2,   "Capture Buffer Overflow"},
		{3,   "Overdriven"},
		{4,   "Underdriven"},
		{6,   "Trigger Timeout"},
		{7,   "Acquisition Error"},
		{8,   "Sync Error"},
		{9,   "Uncal"},
		{15,  "Reference Frequency Error"},
		{16,  "RF Not Available"},
		{17,  "RF Level not Settled"},
		{18,  "RF Frequency not Settled"},
		{19,  "Call not Established"},
		{20,  "Call Type not Usable"},
		{21,  "Call Lost"},
		{23,  "Missing Option"},
		{24,  "Invalid RF Setting"},
		{26,  "Resource Conflict"},
		{27,  "No Sensor Connected"},
		{28,  "Unexpected Parameter Change"},
		{30,  "File not Found"},
		{31,  "No DTM reply"},
		{32,  "ACL Disconnected"},
		{40,  "ARB File CRC Error"},
		{42,  "ARB Header Tag Invalid"},
		{43,  "ARB Segment Overflow"},
		{44,  "ARB File not Found"},
		{45,  "ARB Memory Overflow"},
		{46,  "ARB Sample Rate out of Range"},
		{47,  "ARB Cycles out of Range"},
		{50,  "Startup Error"},
		{51,  "No Reply"},
		{52,  "Connection Error"},
		{53,  "Configuration Error"},
		{54,  "Filesystem Error"},
		{60,  "Invalid RF-Connector Setting"},
		{93,  "OCXO Oven Temperature too low"},
		{101, "Firmware Error"},
		{102, "Unidentified Error"},
		{103, "Parameter Error"},
		{104, "Not Functional"},
		{0, NULL}
	};

	(void)RsCore_LockSession(instrSession);

	for (cnt = 0; reliabilityTable[cnt].string; cnt++)
	{
		if (reliabilityTable[cnt].value == reliabilityCode)
		{
			sprintf(reliabilityMessage, "%s", reliabilityTable[cnt].string);
			if (reliabilityCode)
				error = RSCMW_ERROR_RELIABILITY_INDICATOR;
			found = VI_TRUE;
		}
	}

	if (!found)
	{
		sprintf(reliabilityMessage, "Unrecognized reliability indicator! Report the problem to R&S support!");
		error = RS_WARN_UNKNOWN_INSTRUMENT_STATUS;
	}

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rscmwdau_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rscmw_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
	return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rscmw_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	rsSession->multiThreadLocking = multiThreadLocking;

Error:
	return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_GetError(ViSession instrSession,
                                 ViStatus  *errorCode,
                                 ViInt32   bufferSize,
                                 ViChar    description[])
{
	ViStatus error = VI_SUCCESS;

	// Lock Session - do not jump to the end, even with invalid instrSession the function continues further
	(void)(RsCore_LockSession(instrSession));

	// Test for nulls and acquire error data
	viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

	checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rscmw_error_message, errorCode, bufferSize, description)
    );

Error:
	(void)RsCore_UnlockSession(instrSession); // IGNORE: ERROR!!! Missing Lock
	return error;
}

ViStatus _VI_FUNC rscmw_ClearError(ViSession instrSession)
{
	return RsCore_ClearErrorInfo(instrSession);
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rscmw_WriteInstrData(ViSession instrSession,
                                       ViString  writeBuffer)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	error = RsCore_Write(instrSession, writeBuffer);

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

ViStatus _VI_FUNC rscmw_ReadInstrData(ViSession instrSession,
                                      ViInt32   numBytes,
                                      ViChar    rdBuf[],
                                      ViUInt32  *bytesRead)
{
	ViStatus error = VI_SUCCESS;
	ViChar   *responseString = NULL;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_ReadViStringUnknownLength(instrSession, &responseString));
	checkErr(RsCore_CopyToUserBufferAsciiData(rdBuf, numBytes, responseString));
	*bytesRead = (ViUInt32)strlen(responseString);

Error:
	if (responseString)
		free(responseString);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function is used to read data from the host computer and
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rscmw_WriteFromFileToInstrument(ViSession instrSession,
                                                  ViString  source,
                                                  ViString  destination)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
	checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function is used to read data from the instrument and write
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rscmw_ReadToFileFromInstrument(ViSession instrSession,
                                                 ViString  source,
                                                 ViString  destination)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
	checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmw_WriteCommandWithOPCSync(ViSession instrSession,
                                                ViString  writeBuffer,
                                                ViUInt32  timeout)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN Sends a query to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for a response that can take a long
/// HIFN time to complete e.g. self-alignment, calibration or self-test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data
/// HIPAR readBuffer/that was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument
/// HIPAR numBytesRead/and stored in the Read Buffer.
ViStatus _VI_FUNC rscmw_QueryWithOPCSync(ViSession instrSession,
                                         ViString  writeBuffer,
                                         ViUInt32  timeout,
                                         ViInt32   bufferSize,
                                         ViChar    readBuffer[],
                                         ViUInt32  *numBytesRead)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(writeBuffer), 1, 2147483647),
    	2, "Write Buffer (string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, readBuffer),
    	5, "Read Buffer");

	checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, writeBuffer, timeout, bufferSize, readBuffer, (ViInt32*)numBytesRead));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_QueryViBoolean(ViSession instrSession,
                                       ViString  cmd,
                                       ViBoolean *value)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(cmd), 1, 2147483647), 2,
        "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViBoolean(instrSession, cmd, value));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_QueryViInt32(ViSession instrSession,
                                     ViString  cmd,
                                     ViInt32   *value)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(cmd), 1, 2147483647), 2,
        "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_QueryViReal64(ViSession instrSession,
                                      ViString  cmd,
                                      ViReal64  *value)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(cmd), 1, 2147483647), 2,
        "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViReal64(instrSession, cmd, value));

	checkErr(rscmw_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_QueryViString(ViSession instrSession,
                                      ViString  cmd,
                                      ViInt32   bufferSize,
                                      ViChar    _VI_FAR value[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   *pbuffer = NULL;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(cmd), 1, 2147483647), 2,
        "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

	checkErr(rscmw_CheckStatus(instrSession));

	checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
	if (pbuffer)
		free(pbuffer);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function sets OPC timeout value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/Timeout value
ViStatus _VI_FUNC rscmw_SetOPCTimeout(ViSession instrSession,
                                      ViUInt32  timeout)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, (ViInt32)timeout),
    		3, "Timeout");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function gets OPC timeout value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/Timeout value
ViStatus _VI_FUNC rscmw_GetOPCTimeout(ViSession instrSession,
                                      ViUInt32  *timeout)
{
	ViStatus error = VI_SUCCESS;
	ViInt32  intTimeout;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(rscmw_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, &intTimeout),
			3, "Timeout");

	*timeout = (ViUInt32)intTimeout;

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: rscmw_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rscmw_close(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_ViClose(instrSession));

Error:
	(void)RsCore_Dispose(instrSession);
	(void)RsCore_UnlockSession(instrSession);
	return error;
}
